import { Subject } from 'rxjs';
import { scan, map, pluck, share, switchMap, startWith } from 'rxjs/operators';
import { __assign, __read } from 'tslib';

function createUniqueId(object, length) {
    if (length === void 0) { length = 10; }
    var id;
    for (var i = 0; i === 0 || object.hasOwnProperty(id); i++) {
        id = Math.random().toFixed(36).substring(2, 2 + length);
    }
    return id;
}

function addOne(store, entity) {
    if (!entity) {
        return store;
    }
    addToStore(store, entity);
    return store;
}
function addAll(payload) {
    return payload.reduce(function (store, entity) {
        addToStore(store, entity);
        return store;
    }, { entities: {}, ids: [] });
}
function addMany(store, payload) {
    payload.forEach(function (entity) {
        addToStore(store, entity);
    });
    return store;
}
function removeOne(store, id) {
    if (store.entities[id]) {
        delete store.entities[id];
        store.ids.splice(store.ids.indexOf(id), 1);
    }
    return store;
}
function removeAll() {
    return { entities: {}, ids: [] };
}
function updateOne(store, update) {
    var entity = store.entities[update.id];
    store.entities[update.id] = __assign({}, entity, update.payload);
    return store;
}
function safeId(entity, entities) {
    return entity['id'] ? encodeURIComponent(entity['id']) : createUniqueId(entities);
}
function addToStore(store, entity) {
    var id = safeId(entity, store.entities);
    if (store.entities[id]) {
        return store;
    }
    delete entity['id'];
    store.entities[id] = entity;
    store.ids.push(id);
    return store;
}

/**
 * Reactive crud store
 *
 * This store targets the creation of a very simple store
 * to keep your entities in. It provides modification functions
 * for adding, adding all, removing one by id, remove all,
 * update one.
 *
 * The read functions return observables to read the stored
 * entities in an object or array way. Also you can get a
 * single entity by an id.
 *
 * ## Example
 *
 * ```ts
 * const numberStore = new Store<number>();
 * numberStore.getAll().subscribe(console.log);
 * numberStore.add(8);
 * numberStore.add(3);
 * numberStore.removeAll();
 * // logs ---> [8] ---> [8, 3] ---> []
 * ```
 */
var Store = /** @class */ (function () {
    function Store() {
        this.actions$ = new Subject();
        this.store$ = this.createStore();
        this.entities$ = this.store$.pipe(pluck('entities'));
    }
    Store.prototype.createStore = function () {
        return this.actions$.pipe(scan(function (store, action) {
            switch (action.type) {
                case 'add': return addOne(store, action.payload);
                case 'addMany': return addMany(store, action.payload);
                case 'addAll': return addAll(action.payload);
                case 'update': return updateOne(store, action.payload);
                case 'remove': return removeOne(store, action.payload);
                case 'removeAll': return removeAll();
            }
        }, { entities: {}, ids: [] }), startWith({ entities: {}, ids: [] }), share());
    };
    Store.prototype.add = function (entity) { this.actions$.next({ type: 'add', payload: entity }); };
    Store.prototype.addMany = function (entities) { this.actions$.next({ type: 'addMany', payload: entities }); };
    Store.prototype.addAll = function (entities) { this.actions$.next({ type: 'addAll', payload: entities }); };
    Store.prototype.update = function (changes) { this.actions$.next({ type: 'update', payload: changes }); };
    Store.prototype.remove = function (id) { this.actions$.next({ type: 'remove', payload: id }); };
    Store.prototype.removeAll = function () { this.actions$.next({ type: 'removeAll' }); };
    /**
     * Get one entity filtered by id
     * @param id
     */
    Store.prototype.getOne = function (id) {
        return this.entities$.pipe(pluck(id), map(function (entity) { return (entity ? __assign({ id: id }, entity) : undefined); }));
    };
    /**
     * Get one item by a maybe changing variable. This for example can be useful
     * when getting data by router paramters
     * @param id$
     */
    Store.prototype.getOneDynamic = function (id$) {
        var _this = this;
        return id$.pipe(switchMap(function (id) { return _this.getOne(id); }));
    };
    /**
     * Get all entities in an array including the id property
     */
    Store.prototype.getAll = function () {
        return this.entities$.pipe(map(function (entities) {
            return Object.entries(entities).map(function (_a) {
                var _b = __read(_a, 2), id = _b[0], entity = _b[1];
                return (__assign({ id: id }, entity));
            });
        }));
    };
    /**
     * Get all entities as an object with keys
     */
    Store.prototype.getEntities = function () {
        return this.entities$;
    };
    /**
     * Return all ids
     */
    Store.prototype.getIds = function () {
        return this.store$.pipe(pluck('ids'));
    };
    return Store;
}());

/**
 * Generated bundle index. Do not edit.
 */

export { Store };

//# sourceMappingURL=reactive-entity-store.js.map