import { Observable } from 'rxjs';
import { Update, Entities } from './interfaces';
/**
 * Reactive crud store
 *
 * This store targets the creation of a very simple store
 * to keep your entities in. It provides modification functions
 * for adding, adding all, removing one by id, remove all,
 * update one.
 *
 * The read functions return observables to read the stored
 * entities in an object or array way. Also you can get a
 * single entity by an id.
 *
 * ## Example
 *
 * ```ts
 * const numberStore = new Store<number>();
 * numberStore.getAll().subscribe(console.log);
 * numberStore.add(8);
 * numberStore.add(3);
 * numberStore.removeAll();
 * // logs ---> [8] ---> [8, 3] ---> []
 * ```
 */
export declare class Store<T> {
    private store$;
    private actions$;
    private entities$;
    constructor();
    private createStore;
    add(entity: T): void;
    addMany(entities: T[]): void;
    addAll(entities: T[]): void;
    update(changes: Update): void;
    remove(id: string): void;
    removeAll(): void;
    /**
     * Get one entity filtered by id
     * @param id
     */
    getOne(id: string): Observable<T | undefined>;
    /**
     * Get one item by a maybe changing variable. This for example can be useful
     * when getting data by router paramters
     * @param id$
     */
    getOneDynamic(id$: Observable<string>): Observable<T | undefined>;
    /**
     * Get all entities in an array including the id property
     */
    getAll(): Observable<T[]>;
    /**
     * Get all entities as an object with keys
     */
    getEntities(): Observable<Entities<T>>;
    /**
     * Return all ids
     */
    getIds(): Observable<string[]>;
}
